/*
 * Copyright (c) 1999,2000 Arjan van de Ven
 * Copyright (c) 2000      Alessandro Rubini (rubini@linux.it)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * Declaration for protocol-specific data structures
 */

#if 0
/*
 * Request codes
 */
/* Read request */
#define TFTP_RRQ  1
/* Write request */
#define TFTP_WRQ  2
/* Data message */
#define TFTP_DAT  3 
/* Ack message */
#define TFTP_ACK  4 
/* Error message */
#define TFTP_ERR  5

/* 
 * Error codes
 */
/* Not defined, see error string */
#define TFTP_ERR_NOTDEF 0
/* File not found */
#define TFTP_ERR_FNOTF  1
/* Access violation */
#define TFTP_ERR_ACCVIO 2
/* Allocation exceeded */
#define TFTP_ERR_EXCEED 3
/* Illegal operation */
#define TFTP_ERR_ILLOP  4
/* Unknown port */
#define TFTP_ERR_UNKPORT 5
/* File exists */
#define TFTP_ERR_FEXISTS 6
/* No such user */
#define TFTP_ERR_NOUSER 7

/* 
 * Request message
 */
struct tftp_req_body {
    /* filename + mode */
    __u8 data[1];
};

/*
 * Data message 
 */
struct tftp_data_body {
    /* Block number */
    __u8 blockno[2];
    /* Data */
    __u8 data[1];
};

/*
 * Ack message 
 */
struct tftp_ack_body {
    /* Block number */
    __u8 blockno[2];
};

/*
 * Error message 
 */
struct tftp_err_body {
    /* Error number */
    __u8 ecode[2];
    /* Error message */
    __u8 emsg[1];
};

/*
 * Union representing message bodies
 */
union tftp_msg_body {
    struct tftp_req_body  req_body;
    struct tftp_dat_body  dat_body;
    struct tftp_ack_body  ack_body;
    struct tftp_err_body  err_body;
};

/*
 * tftp message . BYTE ALIGNED !!
 */
struct tftp_msg {
    /* Header */
    __u8 opcode[2];
    /* Body */
    union tftp_msg_body;
}

/* 
 * Useful macros
 */
/* Message opcode given pointer to message */
tftp_msg_opcode(m) ((m)->opcode[0] * 256 + (m)->opcode[1])
/* Pointer to filename given pointer to request message */
/tftp_msg_filename(m) ((m)->data)
/* Pointer to mode given pointer to request message */
tftp_msg_mode(m) ((m)->data[strlen(m->data)+1])

#endif /* 0 */

/*
 * State of transfer
 */
enum xfer_status {
    REQRECV,
    SENDING,
    RETX,
    DONE
};

/* Max length of filename */
#define TFTP_MAX_FPATHLEN 512


/** 
 * tftp request structure 
 * @buffer: pointer to input message
 * @opcode: request code: RRQ, WRQ, ...
 * @filename: target file name
 * @mode: tftp transfer mode (netascii or octet)
 * @req_end: pointer to last byte of incoming request.
 * @errstatus: request error status
 * @filp: pointer to file data structure
 * @socket: pointer to socket through which transaction will take place
 * @xstat: request status (REQRECV, SENDING, RETX, DONE...)
 * @readfun: pointer to read and translate function
 * @offset: current file offset
 * @pending_cr: true is a carriage return must be added as the first
 *              char of the next 512 bytes buffer
 * @open_mode: file open mode.
 *
 * This structure holds the whole data concerning a TFTP request
 */
struct tftp_req {
    char *buffer;
    short opcode;
    char filename[TFTP_MAX_FPATHLEN];
    char *mode;
    char *req_end;
    int  errstatus;
    struct file *filp;
    struct socket *socket;
    struct sockaddr_in from;
    enum xfer_status xstat;
    int (*readfun)(struct tftp_req *, char *buffer);
    loff_t offset;
    char pending_cr;
    unsigned int open_mode;
    
};


#if 0 /* Just for a reference */
/*
 * Error codes.
 */
#define	EUNDEF		0		/* not defined */
#define	ENOTFOUND	1		/* file not found */
#define	EACCESS		2		/* access violation */
#define	ENOSPACE	3		/* disk full or allocation exceeded */
#define	EBADOP		4		/* illegal TFTP operation */
#define	EBADID		5		/* unknown transfer ID */
#define	EEXISTS		6		/* file already exists */
#define	ENOUSER		7		/* no such user */
#endif 








